/**
 * MyModel.cpp
 * Created on April 22, 2003
 */

#include "modelerview.h"
#include "modelerapp.h"
#include "modelerdraw.h"
#include <FL/gl.h>

// This is a list of the controls for our model
enum MyModelControls {
    MODELSIZE, LEGWIDTH, LEGHEIGHT, ANIMATION, NUMCONTROLS,
};

// To make a MyModel, we inherit off of ModelerView
class MyModel : public ModelerView {

public:
    
	MyModel( int x, int y, int w, int h, char *label ) : ModelerView( x, y, w, h, label ) {}

    virtual void draw();

private:


};

/**
 */
ModelerView* createMyModel( int x, int y, int w, int h, char *label ) { 

    return new MyModel( x,y,w,h,label ); 
}

/**
 */
#define VAL(x) (ModelerApplication::Instance()->GetControlValue(x))

/**
 */
void drawLeg() {
	
	glPushMatrix();

		drawBox( 0.125, 0.5, 0.125 );
		glTranslated( 0, -0.5, 0 );
		drawBox( 0.125, 0.5, 0.125 );

	glPopMatrix();
}

/**
 */
void MyModel::draw() {

    ModelerView::draw();

	// draw the floor
	setAmbientColor(.1f,.1f,.1f);
	setDiffuseColor(.5f,.5,0);
	
	glPushMatrix();

		// Draw the floor
		glTranslated( -2.5, 0, -2.5);
		drawBox( 5, 0.01f, 5 );
		

		glPushMatrix();
			
			// Draw the body
			glTranslated( 2.5, 1, 2.5 );
			drawBox( 1, 1, 2 );

			// Draw the legs
			glTranslated( 0, -0.5, 0 );
			drawLeg();

			glTranslated( 1 - 0.125, 0, 0 );
			drawLeg();

			glTranslated( 0, 0, 2 - 0.125 );
			drawLeg();

			glTranslated( -1 + 0.125, 0, 0 );
			drawLeg();

		glPopMatrix();

	glPopMatrix();
	
}

int main() {
	
	ModelerControl controls[NUMCONTROLS];
	controls[MODELSIZE] = ModelerControl("Model Size", 0.25, 3, 0.1f, 1);
	controls[LEGWIDTH] = ModelerControl("Leg Width", 0.25, 3, 0.1f, 1);
	controls[LEGHEIGHT] = ModelerControl("Leg Height", 0.25, 3, 0.1f, 1);
	
	controls[ANIMATION] = ModelerControl("Animation", 1, 30, 1, 1);
	
    // Initialize the modeler application with your model and the
    // appropriate array of controls.
    ModelerApplication::Instance()->Init(&createMyModel, controls, NUMCONTROLS);


	// make sure we give back the memory to older OSs that don't 
	// clear your memory pool after shutdown.
    int Result = ModelerApplication::Instance()->Run();
	delete ModelerApplication::Instance();
	return Result;

}