// Unicorn model.  
// Author: Eric & Brankica

#include "modelerview.h"
#include "modelerapp.h"
#include "modelerdraw.h"
#include "modelerdraw2.h"
#include <FL/gl.h>

// This is a list of the controls
// Used to access the values of the controls from the user interface.
enum UnicornModelControls
{ 
    XPOS, YPOS, ZPOS, ANIMATION, NUMCONTROLS,
};

bool animationIncrementing = true;

// To make a UnicornModel, we inherit off of ModelerView
class UnicornModel : public ModelerView 
{
public:
    // Constructor for the unicorn model
    UnicornModel(int x, int y, int w, int h, char *label) 
        : ModelerView(x,y,w,h,label) {}
    virtual void draw();
};

// Creator function
// (mostly because of nasty API stuff that we'd rather stay away from)
ModelerView* createUnicornModel(int x, int y, int w, int h, char *label)
{ 
    return new UnicornModel(x,y,w,h,label); 
}

// Instance of the application as a macro
#define VAL(x) (ModelerApplication::Instance()->GetControlValue(x))

// override the draw() method of ModelerView 
void UnicornModel::draw()
{
    // This call takes care of a lot of the nasty projection 
    // matrix stuff.  
    ModelerView::draw();

	if ( ModelerApplication::Instance()->GetAnimating() ) {
		float amount = 1.0f;
		if ( animationIncrementing )
			ModelerApplication::Instance()->SetControlValue( ANIMATION, VAL(ANIMATION) + amount );
		else
			ModelerApplication::Instance()->SetControlValue( ANIMATION, VAL(ANIMATION) - amount );

		if ( VAL(ANIMATION) == 40 || VAL(ANIMATION) == 1 )
			animationIncrementing = !animationIncrementing;
	}

	// draw the moon
	setDiffuseColor(1,1,1.4f);
	glPushMatrix();
		glTranslated(-2,4,-4);
		drawMoon( "moon.bmp", 64, 64 );
	glPopMatrix();



	// draw the floor
	setAmbientColor(.1f,.1f,.1f);
	setDiffuseColor(.5f,.5,0);
	glPushMatrix();
		glTranslated(-2.5,0,-2.5);
		drawBox(5,.01f,5, "grass.bmp", 64, 64 );

		// draw the body
		setDiffuseColor(1.2f,1,1);
		glTranslated(VAL(XPOS),VAL(YPOS),VAL(ZPOS));
		glRotated(-VAL(ANIMATION)*.2,.0,.0,1.0);
		glTranslated(-.5,2,-.5);	
		glPushMatrix();
			drawBox(1,.8,.5, "coat.bmp", 32, 32 );

			// draw the front right leg
			glRotated(-VAL(ANIMATION)*1.6,.0,.0,1.0);
			glTranslated(.1,-.8,.1);
			// upper leg
			drawBox( .15,1.3,.15, "coat.bmp", 32, 32 );
			// lowerleg 
			glRotated(VAL(ANIMATION)*2.3,.0,.0,1.0);
			glTranslated( 0,-1.2,0 );
			drawBox( .15,1.2,.15, "coat.bmp", 32, 32 );
		glPopMatrix();

		// draw the front left leg
		glPushMatrix();
			// upper leg 
			glRotated(-VAL(ANIMATION)*.8,.0,.0,1.0);
			glTranslated(.1,-.8,.27);
			drawBox(.15,1,.15, "coat.bmp", 32, 32 );
			// lower leg
			glRotated(VAL(ANIMATION)*1.8,.0,.0,1.0);
			glTranslated(0,-1.2,0);
			drawBox(.15,1.2,.15, "coat.bmp", 32, 32 );
		glPopMatrix();

		// draw the back right leg 
		glPushMatrix();
			glRotated(VAL(ANIMATION)*.2,.0,.0,1.0);
			glTranslated(.7,-.8,.1);
			// upper leg
			drawBox( .15,1.3,.15, "coat.bmp", 32, 32 );
			// lower leg 
			glRotated(VAL(ANIMATION)*.3,.0,.0,1.0);
			glTranslated(0,-1.2,0); 
			drawBox(.15,1.2,.15, "coat.bmp", 32, 32 );		

		glPopMatrix();

		// draw the back left leg 
		glPushMatrix();
			glTranslated(.7,-.8,.27);
			// upper leg
			drawBox( .15,1.3,.15, "coat.bmp", 32, 32 );
			// lower leg 
			glRotated(VAL(ANIMATION)*.4,.0,.0,1.0);
			glTranslated(0,-1.2,0); 
			drawBox(.15,1.2,.15, "coat.bmp", 32, 32 );		
		glPopMatrix();

		glPushMatrix();
			// draw the neck
			glRotated(VAL(ANIMATION)/2.5,.0,.0,1.0);
			glTranslated(.2,.6,.15);
			drawBox(.2,1.4,.2, "coat.bmp", 32, 32 );
			// draw the head
			glTranslated(-0.3,1.1,0);
			glRotated(15,.0,.0,1.0);
			glRotated(VAL(ANIMATION)/3,.0,.0,1.0);
			drawBox(.5,.3,.2, "coat.bmp", 32, 32 );

			// draw ears
			glTranslated(.45,.3,.17);
			glRotated(-26,.0,1.0,.0);
			drawEar( "coat.bmp", 32, 32 );
			glRotated(52,.0,1.0,.0);
			glTranslated(.061,.0,-.123);
			drawEar( "coat.bmp", 32, 32 );
		
			// draw the mane
			glTranslated(-.02,.0,.011);
			glRotated(-26,.0,1.0,.0);
			glRotated(-10, .0,.0,1.0);
			glRotated(-VAL(ANIMATION)/5,.0,.0,1.0);
			drawBox(.2,-1,.1, "hair.bmp", 32, 32 );
		
			// draw the corn
			glTranslated(-.1,.0,.05);
			glRotated(-90,1.0,.0,.0);
			glRotated(-15,.0,1.0,.0);
			drawCorn( "horn.bmp", 16, 16 );
	
		glPopMatrix();

		// draw the tail
		glPushMatrix();
			glTranslated(1,.82,.2);
			glRotated(-170,.0,.0,1.0);
			drawBox(.2,1.7,.1, "hair.bmp", 32, 32 );
		glPopMatrix();	



	glPopMatrix();


}

int main()
{
	// Initialize the controls
    ModelerControl controls[NUMCONTROLS];
	controls[XPOS]      = ModelerControl("X Position", .5, 4.5, .01f, 2.5f);
	controls[YPOS]      = ModelerControl("Y Position",  0,   1, .01f,    0);
	controls[ZPOS]      = ModelerControl("Z Position", .5,   5, .01f, 2.5f);
	controls[ANIMATION] = ModelerControl("Animation",   1,  40, .01f,    1);

    // Initialize the modeler application
    ModelerApplication::Instance()->Init(&createUnicornModel, controls, NUMCONTROLS);


	// make sure we give back the memory to older OSs that don't 
	// clear your memory pool after shutdown.
    int Result = ModelerApplication::Instance()->Run();
	delete ModelerApplication::Instance();
	return Result;

}

